/**************************************************************************
*  PCVideo Desk switcher for VirtuaWin                                    *
*     by Yudong Yang                                                      *
*     yangyd@yahoo.com                                                    *
*                                                                         *
***************************************************************************
*  Copyright (C) 2000, Yang Yudong, All rights reserved.                  *
*  This file is a module for VirtuaWin by Johan Piculell.                 *
*  If you use this software, you agree to the following:                  *
*  This program package is purely experimental, and is licensed "as is".  *
*  Permission is granted to use, modify, and distribute this program      *
*  without charge for any purpose, provided this license/ disclaimer      *
*  notice appears in the copies.  No warranty or maintenance is given,    *
*  either expressed or implied.  In no event shall the author(s) be       *
*  liable to you or a third party for any special, incidental,            *
*  consequential, or other damages, arising out of the use or inability   *
*  to use the program for any purpose (or the loss of data), even if we   *
*  have been advised of such possibilities.  Any public reference or      *
*  advertisement of this source code should refer to it as Yang Yudong's  *
*  orignal.                                                               *
**************************************************************************/

/*
* Simple module skeleton for VirtuaWin. These are the minimal requirements.
* It is a simple application with a hidden window that receives messages from virtuawin
* Look in Messages.h to see what can be sent to and from VirtuaWin
* 
* Note that the classname must be the same as the filename including the '.exe'
*
* Created by Johan Piculell 1999
* VirtuaWin@iname.com
* Feel free to use in any way you like(well, almost)
*/

// #define _BLINK_ICON  // uncommont this to enable blink of the desk top icon
#define _BLINK_SPEED  500  // the delay between blinks in ms

#define WINVER 0x0400

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "Messages.h"
#include "resource.h"

#define  WM_SHELLNOTIFY  4000

HINSTANCE hInst;   // Instance handle
HWND hwndMain;	   // Main window handle
HWND vwHandle;     // Handle to VirtuaWin

const char *szAPPClass= "pcvideovws.exe";

/* Main message handler */
LRESULT CALLBACK MainWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);

/* Initializes the window */ 
static BOOL InitApplication(void)
{
	WNDCLASS wc;
	
	memset(&wc, 0, sizeof(WNDCLASS));
	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)MainWndProc;
	wc.hInstance = hInst;
	/* IMPORTANT! The classname must be the same as the filename since VirtuaWin uses 
	this for locating the window */
	wc.lpszClassName = szAPPClass;
	
	if (!RegisterClass(&wc))
		return 0;
	
	return 1;
}

static BOOL CenterWindow (HWND hwndChild)
{
	RECT    rChild, rParent, rWorkArea;
	int     wChild, hChild, wParent, hParent;
	int     xNew, yNew;
	BOOL  bResult;
	HWND hwndParent;
	
	hwndParent=GetDesktopWindow();
	// Get the Height and Width of the child window
	GetWindowRect (hwndChild, &rChild);
	wChild = rChild.right - rChild.left;
	hChild = rChild.bottom - rChild.top;
	
	// Get the Height and Width of the parent window
	GetWindowRect (hwndParent, &rParent);
	wParent = rParent.right - rParent.left;
	hParent = rParent.bottom - rParent.top;
	
	// Get the limits of the 'workarea'
	bResult = SystemParametersInfo(
		SPI_GETWORKAREA,  // system parameter to query or set
		sizeof(RECT),
		&rWorkArea,
		0);
	if (!bResult) {
		rWorkArea.left = rWorkArea.top = 0;
		rWorkArea.right = GetSystemMetrics(SM_CXSCREEN);
		rWorkArea.bottom = GetSystemMetrics(SM_CYSCREEN);
	}
	
	// Calculate new X position, then adjust for workarea
	xNew = rParent.left + ((wParent - wChild) /2);
	if (xNew < rWorkArea.left) {
		xNew = rWorkArea.left;
	} else if ((xNew+wChild) > rWorkArea.right) {
		xNew = rWorkArea.right - wChild;
	}
	
	// Calculate new Y position, then adjust for workarea
	yNew = rParent.top  + ((hParent - hChild) /2);
	if (yNew < rWorkArea.top) {
		yNew = rWorkArea.top;
	} else if ((yNew+hChild) > rWorkArea.bottom) {
		yNew = rWorkArea.bottom - hChild;
	}
	
	// Set it, and return
	return SetWindowPos (hwndChild, NULL, xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}


LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) {
	case WM_INITDIALOG:
		ShowWindow (hDlg, SW_HIDE);
		CenterWindow (hDlg);
		ShowWindow (hDlg, SW_SHOW);
		return (TRUE);
	case WM_LBUTTONUP:
		EndDialog(hDlg, TRUE);
		return (TRUE);
	}
    return FALSE;
}


LRESULT CALLBACK MainWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static NOTIFYICONDATA IconData;
	static HICON wnd_icon[9];
	static HICON hiVWS;
	static int currDesk;
	static MENUITEMINFO mymenu;
	static BOOL bVWSicon=FALSE;
	
#define ICON_TIMER   1
	
	switch (msg) {
	case WM_CREATE:
		{
			int i;
			
			// load icons
			for(i=0; i<9; i++)	wnd_icon[i] = LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON1+i));
			hiVWS = LoadIcon(hInst, MAKEINTRESOURCE(IDI_VWS));
			// taks bar icon
			IconData.cbSize = sizeof(IconData);
			IconData.hWnd = hwnd;
			IconData.uID = 710708;
			IconData.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
			IconData.uCallbackMessage = WM_SHELLNOTIFY;
			IconData.hIcon = hiVWS;
			strcpy(IconData.szTip, "PCVideo Window Switcher for VirtuaWin");
			Shell_NotifyIcon(NIM_ADD, &IconData);
			IconData.uFlags = NIF_ICON; // Others doesn't change afterwards
			
			return 0;
		}
	case WM_DESTROY:
		{
			int i;
			for(i=0; i<9; i++) DestroyIcon(wnd_icon[i]);
			DestroyIcon(hiVWS);
			PostQuitMessage(0);
			break;
		}
	case WM_SHELLNOTIFY:
		{
			if (lParam == WM_LBUTTONUP || lParam == WM_RBUTTONUP) 
			{
				HMENU hPopupMenu;
				POINT pnt;
				char szMenu[256];
				int i, wmId;

				// create menu ??? How can I get current number of desks VurtuaWin does not give such info?
				hPopupMenu = CreatePopupMenu();
				mymenu.cbSize = sizeof(MENUITEMINFO);
				mymenu.fMask = MIIM_TYPE | MIIM_ID;
				mymenu.fType = MFT_STRING;
				mymenu.dwTypeData = szMenu;
				for(i=0; i<9; i++)
				{
					mymenu.wID = IDM_WINDOW1+i;
					sprintf(szMenu, "Goto Desk %d", i+1);
					InsertMenuItem(hPopupMenu, i, TRUE, &mymenu);
				}
				mymenu.wID = IDM_ABOUT;
				mymenu.dwTypeData = "About";
				InsertMenuItem(hPopupMenu, 9, TRUE, &mymenu);
				// show menu
				GetCursorPos((LPPOINT)&pnt);
				SetForegroundWindow(hwnd); // this is must else your menu will not be closed automatically
				wmId = TrackPopupMenu (hPopupMenu, TPM_RETURNCMD|TPM_RIGHTBUTTON, pnt.x-2, pnt.y-2, 0, hwnd, NULL);
				// close menu
				DestroyMenu(hPopupMenu);

				if(wmId >= IDM_WINDOW1 && wmId <= IDM_WINDOW9 && hPopupMenu) // switch desk
				{
					int toDesk = wmId-IDM_WINDOW1+1;
					PostMessage(vwHandle, VW_CHANGEDESK, toDesk, toDesk); // switch to desk x
				}
				else if (IDM_ABOUT == wmId)
				{
					DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, (DLGPROC)About);
				}
				return 0;
			}
			break;
		}
		
	case MOD_INIT: // This must be taken care of in order to get the handle to VirtuaWin. 
		// The handle to VirtuaWin comes in the wParam 
		vwHandle = (HWND) wParam; // Should be some error handling here if NULL 
#ifdef _BLINK_ICON
        SetTimer(hwnd, ICON_TIMER, _BLINK_SPEED, NULL);
#endif
	case MOD_CFGCHANGE:
		// get current number of  desk
		break;
#ifdef _BLINK_ICON
	case WM_TIMER:
		{
			if(ICON_TIMER == wParam)
			{
				if(bVWSicon) IconData.hIcon = hiVWS, bVWSicon = FALSE;
				else IconData.hIcon = wnd_icon[currDesk], bVWSicon = TRUE;
				Shell_NotifyIcon(NIM_MODIFY, &IconData);
			}
		}
		break;
#endif
		
	case MOD_QUIT: // This must be handeled, otherwise VirtuaWin can't shut down the module 
#ifdef _BLINK_ICON
        KillTimer(hwnd, ICON_TIMER);
#endif
		Shell_NotifyIcon(NIM_DELETE, &IconData);
		PostQuitMessage(0);
		break;
	case MOD_SETUP: // Optional
        DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUT), vwHandle, (DLGPROC)About);
		break;
		
	case MOD_CHANGEDESK:
		{
			currDesk = lParam-1;
#ifdef _BLINK_ICON
			IconData.hIcon = wnd_icon[currDesk];
			Shell_NotifyIcon(NIM_MODIFY, &IconData);
#endif
			break;
		}
		
	default:
		
		return DefWindowProc(hwnd, msg, wParam, lParam);
	}
	
	return 0;
}

/*
* Main startup function
*/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT nCmdShow)
{
	MSG msg;
	hInst = hInstance;
	if (!InitApplication())
		return 0;
	
	// In this example, the window is never shown
	if ((hwndMain = CreateWindow(szAPPClass, 
		"pcvideovws", 
		WS_POPUP,
		CW_USEDEFAULT, 
		0, 
		CW_USEDEFAULT, 
		0,
		NULL,
		NULL,
		hInst,
		NULL)) == (HWND)0)
		return 0;
	
	// main messge loop
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}
